@@ -22,31 +22,41 @@ enum Model {
 }
 
 impl Model {
+    fn detect() -> Result<Self> {
+        if PathBuf::from("/sys/class/power_supply/rk818-usb").exists() {
+            Ok(Model::PinePhonePro)
+        } else if PathBuf::from("/sys/class/power_supply/axp20x-usb").exists() {
+            Ok(Model::PinePhone)
+        } else {
+            bail!("unknown model")
+        }
+    }
+
     // valid values that can be written to input_current_limit
     fn valid_limits(&self) -> &'static [u32] {
-        static PPP: [u32; 10] = [
-            80000, 450000, 850000, 1000000, 1250000, 1500000, 2000000, 2250000, 2500000, 3000000,
+        static PPP: [u32; 9] = [
+            450000, 850000, 1000000, 1250000, 1500000, 2000000, 2250000, 2500000, 3000000,
         ];
-        //    static PP: ?
+        static PP: [u32; 6] = [500000, 900000, 1500000, 2000000, 2500000, 3000000];
         match self {
             Model::PinePhonePro => &PPP,
-            Model::PinePhone => unimplemented!(),
+            Model::PinePhone => &PP,
         }
     }
 
     // return the default input current limit
     fn default_limit(&self) -> u32 {
         match self {
-            Model::PinePhonePro => self.valid_limits()[1],
-            Model::PinePhone => unimplemented!(),
+            Model::PinePhonePro => self.valid_limits()[0],
+            Model::PinePhone => self.valid_limits()[0],
         }
     }
 
     // return the default input current limit
     fn max_limit(&self) -> u32 {
         match self {
-            Model::PinePhonePro => self.valid_limits()[9],
-            Model::PinePhone => unimplemented!(),
+            Model::PinePhonePro => self.valid_limits()[8],
+            Model::PinePhone => self.valid_limits()[5],
         }
     }
 
@@ -84,17 +94,31 @@ struct Device {
 impl Device {
     fn new(model: Model) -> Device {
         let base = PathBuf::from("/sys/class/power_supply");
-        Device {
-            model,
-            kb_current: base.join("ip5xxx-charger/current_now"),
-            kb_voltage: base.join("ip5xxx-charger/voltage_now"),
-            kb_state: base.join("ip5xxx-charger/status"),
-            kb_limit: base.join("ip5xxx-charger/constant_charge_current"),
-            kb_enabled: base.join("ip5xxx-boost/online"),
-            mb_state: base.join("battery/status"),
-            mb_voltage: base.join("battery/voltage_now"),
-            mb_current: base.join("battery/current_now"),
-            mb_limit: base.join("rk818-usb/input_current_limit"),
+        match model {
+            Model::PinePhonePro => Device {
+                model,
+                kb_current: base.join("ip5xxx-charger/current_now"),
+                kb_voltage: base.join("ip5xxx-charger/voltage_now"),
+                kb_state: base.join("ip5xxx-charger/status"),
+                kb_limit: base.join("ip5xxx-charger/constant_charge_current"),
+                kb_enabled: base.join("ip5xxx-boost/online"),
+                mb_state: base.join("battery/status"),
+                mb_voltage: base.join("battery/voltage_now"),
+                mb_current: base.join("battery/current_now"),
+                mb_limit: base.join("rk818-usb/input_current_limit"),
+            },
+            Model::PinePhone => Device {
+                model,
+                kb_current: base.join("ip5xxx-charger/current_now"),
+                kb_voltage: base.join("ip5xxx-charger/voltage_now"),
+                kb_state: base.join("ip5xxx-charger/status"),
+                kb_limit: base.join("ip5xxx-charger/constant_charge_current"),
+                kb_enabled: base.join("ip5xxx-boost/online"),
+                mb_state: base.join("axp20x-battery/status"),
+                mb_voltage: base.join("axp20x-battery/voltage_now"),
+                mb_current: base.join("axp20x-battery/current_now"),
+                mb_limit: base.join("axp20x-usb/input_current_limit"),
+            },
         }
     }
 
@@ -155,7 +179,7 @@ impl FromStr for State {
 #[derive(Debug)]
 struct KeyboardBattery {
     state: State,
-    voltage: i32,
+    voltage: u32,
     current: i32,
     limit: u32,
     enabled: bool,
@@ -164,10 +188,10 @@ struct KeyboardBattery {
 impl KeyboardBattery {
     async fn get(dev: &Device) -> Result<KeyboardBattery> {
         Ok(KeyboardBattery {
-            state: read::<State>(&dev.kb_state).await??,
-            voltage: read::<i32>(&dev.kb_voltage).await??,
-            current: read::<i32>(&dev.kb_current).await??,
-            limit: read::<u32>(&dev.kb_limit).await??,
+            state: read(&dev.kb_state).await??,
+            voltage: read(&dev.kb_voltage).await??,
+            current: read(&dev.kb_current).await??,
+            limit: read(&dev.kb_limit).await??,
             enabled: read::<i32>(&dev.kb_enabled).await?? == 1,
         })
     }
@@ -176,25 +200,50 @@ impl KeyboardBattery {
 #[derive(Debug)]
 struct MainBattery {
     state: State,
-    voltage: i32,
+    voltage: u32,
     current: i32,
     limit: u32,
 }
 
 impl MainBattery {
     async fn get(dev: &Device) -> Result<MainBattery> {
-        let current = read::<i32>(&dev.mb_current).await??;
-        let state = match read::<State>(&dev.mb_state).await?? {
-            State::Full => State::Full,
-            State::Charging if current > 0 => State::Charging,
-            State::Charging => State::Discharging,
-            State::Discharging => State::Discharging
-        };
-        Ok(MainBattery {
-            state, current,
-            voltage: read::<i32>(&dev.mb_voltage).await??,
-            limit: read::<u32>(&dev.mb_limit).await??,
-        })
+        async fn get_state(dev: &Device, current: i32) -> Result<State> {
+            Ok(match read(&dev.mb_state).await?? {
+                State::Full => State::Full,
+                State::Charging if current > 0 => State::Charging,
+                State::Charging => State::Discharging,
+                State::Discharging => State::Discharging,
+            })
+        }
+        match dev.model {
+            Model::PinePhonePro => {
+                let current: i32 = read(&dev.mb_current).await??;
+                Ok(MainBattery {
+                    state: get_state(dev,current).await?,
+                    current,
+                    voltage: read(&dev.mb_voltage).await??,
+                    limit: read(&dev.mb_limit).await??,
+                })
+            }
+            Model::PinePhone => {
+                let limit: u32 = read(&dev.mb_limit).await??;
+                let current_abs: i32 = read(&dev.mb_current).await??;
+                // this hack works around a kernel bug that causes
+                // only abs(current) to be reported. It isnt't
+                // perfect, but it catches the obvious cases.
+                let current = if current_abs > ((limit as i32) + (limit as i32 >> 2)) {
+                    !current_abs
+                } else {
+                    current_abs
+                };
+                Ok(MainBattery {
+                    state: get_state(dev, current).await?,
+                    current,
+                    voltage: read(&dev.mb_voltage).await??,
+                    limit,
+                })
+            }
+        }
     }
 }
 
@@ -235,7 +284,7 @@ async fn step(dev: &Device, kb_charging: &mut bool, last_step: &mut Instant) ->
             }
         }
         State::Full => {
-            if info.kbd.enabled {
+            if info.kbd.enabled && *kb_charging {
                 Action::SetMax
             } else {
                 Action::SetDefault
@@ -252,7 +301,7 @@ async fn step(dev: &Device, kb_charging: &mut bool, last_step: &mut Instant) ->
                     State::Discharging => {
                         let mb = info.mb.current.abs();
                         let kb = info.kbd.current.abs();
-                        if mb >= (kb >> 1) {
+                        if mb > kb {
                             Action::MaybeStepUp
                         } else {
                             Action::Pass
@@ -305,7 +354,7 @@ async fn step(dev: &Device, kb_charging: &mut bool, last_step: &mut Instant) ->
 #[tokio::main(flavor = "current_thread")]
 async fn main() -> Result<()> {
     env_logger::init();
-    let dev = Device::new(Model::PinePhonePro);
+    let dev = Device::new(Model::detect()?);
     let mut kb_charging = false;
     let mut last_step = Instant::now();
     loop {
